/* 
    jgaadb - is a simple C++ class library for ODBC databases.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/
#ifndef DB_COLUMN_H
#define DB_COLUMN_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */


/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/

#define DB_MIN(a,b) (a < b ? a : b)

#ifdef _MSC_VER
#   ifndef SQLLEN
#       define SQLLEN SQLINTEGER
#   endif
#endif

/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

BEGIN_JGAADB_NAMESPACE

typedef char db_tinyint_t;
typedef unsigned char db_utinyint_t;
typedef short db_short_t;
typedef int db_int_t;
typedef unsigned short db_ushort_t;
typedef unsigned int db_uint_t;
typedef char db_char_t;
typedef wchar_t db_wchar_t;
typedef db_char_t * db_cstr_t;
typedef db_wchar_t * db_wstr_t;
typedef char * db_bin_t;
typedef float db_float_t;
typedef double db_double_t;

#ifdef _MSC_VER
    typedef __int64 db_i64_t;
    typedef unsigned __int64 db_u64_t;
#else
    typedef long long db_i64t;
    typedef unsigned long long db_u64_t;
#endif

typedef SQL_TIME_STRUCT db_time_t;
typedef SQL_DATE_STRUCT db_date_t;
typedef SQL_TIMESTAMP_STRUCT db_timestamp_t;

enum DbDataTypesE
{
    DB_TYPE_TINYINT,
    DB_TYPE_UTINYINT,
    DB_TYPE_BIGINT,
    DB_TYPE_UBIGINT,
    DB_TYPE_SHORT,
    DB_TYPE_INT,
    DB_TYPE_USHORT,
    DB_TYPE_UINT,
    DB_TYPE_CHAR, 
    DB_TYPE_VARCHAR,
    DB_TYPE_WCHAR,
    DB_TYPE_VARWCHAR,
    DB_TYPE_DATE,
    DB_TYPE_TIME,
    DB_TYPE_TIMESTAMP,
    DB_TYPE_BIN,
    DB_TYPE_VARBIN,
    DB_TYPE_FLOAT,
    DB_TYPE_DOUBLE,
    DB_TYPE_MONEY,
    DB_TYPE_TEXT,
    DB_TYPE_WTEXT,
    DB_TYPE_BLOB,
    DB_TYPE_DECIMAL,
    DB_TYPE_NUMERIC,
    DB_TYPE_INVALID
};

class DbColumn 
{
public:
    typedef union
    {
        db_tinyint_t db_tinyint;
        db_utinyint_t db_utinyint;
        db_short_t db_short;
        db_int_t db_int;
        db_ushort_t db_ushort;
        db_uint_t db_uint;
        db_char_t db_char; 
        db_wchar_t db_wchar; 
        db_cstr_t db_cstr; 
        db_wstr_t db_wstr; 
        db_bin_t db_bin; 
        db_time_t db_time; 
        db_date_t db_date;
        db_timestamp_t db_timestamp;
        db_i64_t db_i64;
        db_i64_t db_u64;
        db_float_t db_float; 
        db_double_t db_double; 
    } data_t;

    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    DbColumn(const char *name,
        const int colSqlType = 0,
        const size_t colMaxSize = 0,
        const size_t colDigits = 0,
        const bool isNullable = false,
        const DbDataTypesE colType = DB_TYPE_INVALID);

    /**
    * Destructor.
    */
    virtual ~DbColumn();
    
    // OPERATORS
    // OPERATIONS

    void ResetCache();
    std::string GetCreateStatement(DbConnection *pTarget = NULL) const;
    void GetBindBuffer(void **pbuffer, 
        size_t*buf_len, int *c_type, int *sql_type,
        DbConnection *pTarget) 
        throw(std::exception);

    void operator = (const db_int_t& val);
	void operator = (const db_short_t& val);
	void operator = (const db_tinyint_t& val);
	void operator = (const db_i64_t& val);
    void operator = (const char * val);
    void operator = (const wchar_t * val);
	void operator = (const time_t& val);

    // CALLBACK
    // ACCESS

    void * GetPointer() throw(DbException);
    const char *GetAsString() throw(std::exception);
	const int GetAsInt() const throw(std::exception);

    // INQUIRY

    DbDataTypesE GetType() const;
    SQLSMALLINT GetCtype() const throw(std::exception);
    SQLSMALLINT GetSqlType() const throw(std::exception);
    size_t GetBufferLength() const;
    SQLSMALLINT GetDigits() const;
    SQLSMALLINT GetBytesUsed() const;
    const char *GetName() const;
    const char *GetCompatibleName() const;
    const char *GetTypeName() const;
    size_t GetSize() const;
    bool IsStringType() const;
    size_t GetCharWidth() const;
    bool IsNull() const;

    SQLLEN mDataLenUsed2; // Data length returned by other calls, like SqlExecure()
    
protected:
    // Allocate or re-allocate buffer.
    void * Allocate(size_t bytes,
        bool doCopyOldBuffer = false) throw(jgaadb::DbBasicException);
    void Clear();

private:
    friend class DbRecordset;
    int mSqlType; // Native ODBC API data type
    DbDataTypesE mType; 
    size_t mSize; // Max [valid] size
    size_t mByteSize; // Required allocated buffer size
    size_t mDigits;
    bool mIsNullable;
    size_t mAllocatedBytes;
    data_t mData;
    std::string mName;
    std::string mCompatibleName;
    SQLLEN mDataLenUsed; // Data length returned by SQLFetch()
    std::string mCachedString;
    bool mUnknownBufferSize;
    size_t mCharWidth;
    bool mIsStringType;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

END_JGAADB_NAMESPACE

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* DB_COLUMN_H_ */

