/* 
    jgaadb - is a simple C++ class library for ODBC databases.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#ifndef JGAADB_DB_CONNECTION_H
#define JGAADB_DB_CONNECTION_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */


/* LOCAL INCLUDES */


/* FORWARD REFERENCES */



#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

BEGIN_JGAADB_NAMESPACE

class DbConnection 
{
public:
    enum OdbcVersionE
    {
        ODBC_V2,
        ODBC_V3
    };

    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    DbConnection();
    
    /**
    * Destructor.
    */
    virtual ~DbConnection();
    
    // OPERATORS   
    // OPERATIONS

    void Connect(const char *Dsn, int loginTimeout = 10)
        throw(DbException);

    void Close(); // Called by destructor

    void EnumColumns(fieldlist_t& fields,
        const db_c_str_t tableName,
        const db_c_str_t colName = NULL,
        const db_c_str_t catalogName = NULL, 
        const db_c_str_t schemaName = NULL)
        throw (std::exception);

	/** Return a name that can be accepted by the
	  * SQL server, like "File" on MS SQL Server
	  */
	std::string GetValidName(const char *name);

    // CALLBACK
    // ACCESS

    HENV GetHdbc() const;
    HENV GetHenv() const;

    // INQUIRY
    bool IsConnected() const;
    bool IsTable(const db_c_str_t tableName,
        const db_c_str_t catalogName = NULL, 
        const db_c_str_t schemaName = NULL)
        throw (std::exception);
	
	bool QueryInfo(SQLUSMALLINT infoType, 
		std::string& infoBuffer);

	const std::string& GetDbmsName() const;
	const std::string& GetDbmsVersion() const;
	const std::string& GetSqlQuoteChar() const;
	
protected:
private:
    HDBC mHdbc; // Database handle
    HENV mHenv; // Environment handle
    std::string mDsn; // Dsn used to open database connection
    std::string mConnectDsn; // DSN returned from ODBC
	std::string mDbmsName;
	std::string mDbmsVersion;
	std::string mSqlQuoteChar;
    OdbcVersionE mOdbcVersion; // ODBC version supported by driver
    bool mIsConnected;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

inline bool JGAADB_SQLOK(int val)
{
    return ((SQL_SUCCESS == val) 
		|| (SQL_SUCCESS_WITH_INFO == val)
		|| (SQL_NO_DATA == val));
}

END_JGAADB_NAMESPACE

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* JGAADB_DB_CONNECTION_H_ */

