/* 
    jgaadb - is a simple C++ class library for ODBC databases.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include "jgaadb.h"   // class implemented


/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

BEGIN_JGAADB_NAMESPACE

DbException::DbException(DbConnection *pconn,
        int handleType, 
        SQLHANDLE handle,
        const char *moduleName,
        const char *explanation,
        const char *query,
        int line,
        const char *fileName) :
jgaadb::DbBasicException(NULL, fileName, line),
mSqlNativeError(0)
{
    std::ostringstream buf;
    buf << "Database error in module \""
        << moduleName
        << "\" ";

    if (explanation)
        buf << explanation << ' ';

    if (query)
        buf << "Query: \"" << query << "\". ";

    // Try to figure out the actual problem...
    SQLCHAR SQLmsg[512];
	SQLmsg[0] = 0;
    memset(mSqlState, 0, sizeof(mSqlState));

	SQLSMALLINT SQLTextLength = sizeof(SQLmsg);

#if HAVE_SQLGETDIAGREC
	SQLRETURN  status = ::SQLGetDiagRec(handleType,
        handle,
        1,
        (SQLCHAR *)&mSqlState,
		&mSqlNativeError,
        SQLmsg,
        sizeof(SQLmsg) -1,
        &SQLTextLength);
#else
    SQLRETURN  status = ::SQLError(
        (SQL_HANDLE_ENV == handleType) ? handle : NULL,
        (SQL_HANDLE_DBC == handleType) ? handle : NULL,
        (SQL_HANDLE_STMT == handleType) ? handle : NULL,
        (SQLCHAR *)&mSqlState,
		&mSqlNativeError,
        SQLmsg,
        sizeof(SQLmsg) -1,
        &SQLTextLength);
#endif

    switch(status)
    {
    case SQL_SUCCESS:
    case SQL_SUCCESS_WITH_INFO:
        buf << "SQL state = \"" << (const char *)mSqlState << "\" Native Error = \""
            << (int)mSqlNativeError << "\" : \"" << (const char *)SQLmsg << "\"";
        break;
    case SQL_ERROR:
        buf << "SQL error on call to resolve SQL error :/";
        break;
    case SQL_INVALID_HANDLE:
        buf << "Invalid handle when resolving SQL error.";
        break;
#if SQL_NO_DATA
    case SQL_NO_DATA:
        buf << "No error information was returned from the ODBC driver";
        break;
#endif // SQL_NO_DATA
    default:
        buf << "Undocumented return value "
            << status 
            << " from SQLGetDiagRec()";
    }

    mWhy += buf.str();

}// DbException


DbException::~DbException()
{
}// ~DbException


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= CALLBACK   ===================================
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

void DbException::GetSqlState(std::string& state) const
{
	state = "";

	const SQLCHAR *p = mSqlState;
	while(*p)
		state += (char)*p++;
}

SQLINTEGER DbException::GetSqlNativeError() const
{
	return mSqlNativeError;
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////


END_JGAADB_NAMESPACE
