/* 
    jgaadb - is a simple C++ class library for ODBC databases.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/


#include "StdAfx.h"
#include "jgaadb.h"


BEGIN_JGAADB_NAMESPACE

static DbConnection my_conn;

static void dump_table(const DbTable& rtable)
{
    std::cout << "  Table: " 
        << rtable
        << " [" << rtable.GetTypeName() << ']'
        << std::endl;
}

static void query_table(const DbTable& rtable)
{
    int index = 0;
    DbCommand my_cmd(my_conn);
    DbRecordset resultset;
    
    std::string query = "SELECT * FROM ";
    query += rtable.GetName();
    
    my_cmd.Execute(query.c_str(), &resultset);
    
    std::cout << "query: " << query << std::endl << "Columns: ";
    
    for(index = 0; index < resultset.GetNumColumns(); index++)
    {
        std::cout << " >> "
            << resultset[index].GetName()
            << " "
            << resultset[index].GetTypeName()
            << " Buffer length: " 
            << resultset[index].GetBufferLength()
            << " Data length: "
            << resultset[index].GetSize()
            << std::endl;
    }
    
    for(index = 0; (index < 10) && !resultset.IsEof(); index++)
    {
        std::cout << " -- " 
            << index
            << " -- ";

        for(int col = 0; col < resultset.GetNumColumns(); col++)
        {
            std::cout << " | " << resultset[col].GetAsString();
        }
        
        std::cout << std::endl;
        
        resultset.MoveNext();
    }
}

// Test the library. 
void jgaadb_test(const char *dsn)
{
    // Create database connection
    db_table_list_t my_tables;

    try
    {
        std::cout << "Connnection to: " << dsn << std::endl;
        my_conn.Connect(dsn);
    }
    catch(std::exception& ex)
    {
        std::cout << "Caught exception from Connect: "
            << ex.what()
            << std::endl;
        my_conn.Close();
        return;
    }

    std::cout << "Enumerating tables" << std::endl;
    // Enum tables
    try
    {
        DbCommand my_cmd(my_conn);

        my_cmd.EnumTables(my_tables);

        std::for_each(my_tables.begin(), my_tables.end(), 
            dump_table);
    }
    catch(std::exception& ex)
    {
        std::cout << "Caught exception during table enumeration: "
            << ex.what()
            << std::endl;
        my_conn.Close();
        return;
    }

    // Enum cols in each table

    // Output top 10 rows in each table
    
    try
    {
        std::for_each(my_tables.begin(), my_tables.end(), 
            query_table);
    }
    catch(std::exception& ex)
    {
        std::cout << "Caught exception during table queries: "
            << ex.what()
            << std::endl;
        my_conn.Close();
        return;
    }

    my_conn.Close();
}

END_JGAADB_NAMESPACE

